@@ -2,7 +2,7 @@
 #define FVWMRC ".fvwm2rc"
 
 /* Define if gdk-imlib is used */
-#undef IMLIB
+#undef GDK_IMLIB
 
 /* define if GNOME WM hints are enabled */
 #undef GNOME
@@ -16,6 +16,12 @@
 /* Define if rplay library is used.  */
 #undef HAVE_RPLAY
 
+/* Define if Xinerama should be emulated on a single screen. */
+#undef USE_XINERAMA_EMULATION
+
+/* Define if Xinerama library is used. */
+#undef HAVE_XINERAMA
+
 /* Define if stroke library is used. */
 #undef HAVE_STROKE
 
@@ -116,6 +122,24 @@
 #endif
 
 
+/* Old AIX systems (3.2.5) don't define some common keysyms. */
+#undef COMPAT_OLD_KEYSYMDEF
+
+#ifdef COMPAT_OLD_KEYSYMDEF
+#  define XK_Page_Up   XK_Prior
+#  define XK_Page_Down XK_Next
+#endif
+
+
+/* Old libstroke <= 0.4 does not use STROKE_ prefix for constants. */
+#undef COMPAT_OLD_LIBSTROKE
+
+#ifdef COMPAT_OLD_LIBSTROKE
+/* currently we only use one constant */
+#  define STROKE_MAX_SEQUENCE MAX_SEQUENCE
+#endif
+
+
 /**
  * The next few defines are options that are only changed from their values
  * shown here on systems that _don't_ use the configure script.
@@ -140,9 +164,12 @@
 
 @TOP@
 
+#if 0
+/* migo: Commented; this is not removed with autoconf-2.50 anymore. */
 #error The stuff above TOP goes to the top of config.h.in
 #error What appears below BOTTOM goes to the bottom
 #error This text should not appear anywhere!
+#endif
 
 @BOTTOM@
 
