@@ -57,6 +57,7 @@
 
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
+#include "libs/FScreen.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -249,6 +250,7 @@ void setup_class_and_resource(FvwmWindow *tmp_win)
 void setup_wm_hints(FvwmWindow *tmp_win)
 {
   tmp_win->wmhints = XGetWMHints(dpy, tmp_win->w);
+  set_focus_model(tmp_win);
 }
 
 static void destroy_window_font(FvwmWindow *tmp_win)
@@ -547,7 +549,7 @@ void setup_frame_size_limits(FvwmWindow *tmp_win, window_style *pstyle)
 
 Bool setup_window_placement(FvwmWindow *tmp_win, window_style *pstyle)
 {
-  int client_argc;
+  int client_argc = 0;
   char **client_argv = NULL;
   XrmValue rm_value;
   int tmpno1 = -1, tmpno2 = -1, tmpno3 = -1, spargs = 0;
@@ -568,7 +570,8 @@ Bool setup_window_placement(FvwmWindow *tmp_win, window_style *pstyle)
   /* Find out if the client requested a specific desk on the command line. */
   /*  RBW - 11/20/1998 - allow a desk of -1 to work.  */
 
-  if (XGetCommand (dpy, tmp_win->w, &client_argv, &client_argc))
+  if (XGetCommand(dpy, tmp_win->w, &client_argv, &client_argc) &&
+      client_argc > 0 && client_argv != NULL)
   {
     /* Get global X resources */
     MergeXResources(dpy, &db, False);
@@ -589,6 +592,12 @@ Bool setup_window_placement(FvwmWindow *tmp_win, window_style *pstyle)
       }
       pstyle->flags.use_start_on_desk = 1;
     }
+    if (GetResourceString(db, "fvwmscreen", client_argv[0], &rm_value) &&
+	rm_value.size != 0)
+    {
+      SSET_START_SCREEN(*pstyle, FScreenGetScreenArgument(rm_value.addr, 'c'));
+      pstyle->flags.use_start_on_screen = 1;
+    }
     if (GetResourceString(db, "page", client_argv[0], &rm_value) &&
 	rm_value.size != 0)
     {
@@ -953,9 +962,24 @@ void setup_frame_attributes(
   XSetWindowAttributes xswa;
 
   /* Backing_store is controlled on the client, decor_w, title & buttons */
-  xswa.backing_store = pstyle->flags.use_backing_store
-		       ? Scr.use_backing_store : NotUseful;
+  switch (pstyle->flags.use_backing_store)
+  {
+  case BACKINGSTORE_DEFAULT:
+	  xswa.backing_store = tmp_win->initial_backing_store;
+	  break;
+  case BACKINGSTORE_ON:
+	  xswa.backing_store = Scr.use_backing_store;
+	  break;
+  case BACKINGSTORE_OFF:
+  default:
+	  xswa.backing_store = NotUseful;
+	  break;
+  }
   XChangeWindowAttributes(dpy, tmp_win->w, CWBackingStore, &xswa);
+  if (pstyle->flags.use_backing_store == BACKINGSTORE_OFF)
+  {
+    xswa.backing_store = NotUseful;
+  }
   XChangeWindowAttributes(dpy, tmp_win->decor_w, CWBackingStore, &xswa);
   if (HAS_TITLE(tmp_win))
   {
@@ -1301,6 +1325,9 @@ FvwmWindow *AddWindow(Window w, FvwmWindow *ReuseWin)
   setup_class_and_resource(tmp_win);
   setup_wm_hints(tmp_win);
 
+  /* save a copy of the backing store attribute */
+  tmp_win->initial_backing_store = tmp_win->attr.backing_store;
+
   /****** basic style and decor ******/
   /* If the window is in the NoTitle list, or is a transient, dont decorate it.
    * If its a transient, and DecorateTransients was specified, decorate anyway.
@@ -1581,7 +1608,10 @@ void FetchWmProtocols (FvwmWindow *tmp)
     for (i = 0, ap = protocols; i < n; i++, ap++)
     {
       if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
+      {
 	SET_WM_TAKES_FOCUS(tmp, 1);
+	set_focus_model(tmp);
+      }
       if (*ap == (Atom)_XA_WM_DELETE_WINDOW)
 	SET_WM_DELETES_WINDOW(tmp, 1);
     }
@@ -1600,7 +1630,10 @@ void FetchWmProtocols (FvwmWindow *tmp)
       for (i = 0, ap = protocols; i < nitems; i++, ap++)
       {
 	if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
+	{
 	  SET_WM_TAKES_FOCUS(tmp, 1);
+	  set_focus_model(tmp);
+	}
 	if (*ap == (Atom)_XA_WM_DELETE_WINDOW)
 	  SET_WM_DELETES_WINDOW(tmp, 1);
       }
@@ -1863,7 +1896,8 @@ void GetWindowSizeHints(FvwmWindow *tmp)
   {
     fvwm_msg(WARN, "GetWindowSizeHints",
 	     "%s window %#lx has broken (%s) size hints\n"
-	     "Please report this to fvwm-workers@fvwm.org\n"
+	     "Please send a bug report to the application owner,\n"
+	     "\tyou may use fvwm-workers@fvwm.org as a reference.\n"
 	     "hint override = %d\n"
 	     "flags = %lx\n"
 	     "min_width = %d, min_height = %d, "
@@ -1935,6 +1969,22 @@ void free_window_names(FvwmWindow *tmp, Bool nukename, Bool nukeicon)
   return;
 }
 
+
+static void adjust_fvwm_internal_windows(FvwmWindow *tmp_win)
+{
+  extern FvwmWindow *ButtonWindow;
+
+  if (tmp_win == Scr.Hilite)
+    Scr.Hilite = NULL;
+  update_last_screen_focus_window(tmp_win);
+  if (ButtonWindow == tmp_win)
+    ButtonWindow = NULL;
+  restore_focus_after_unmap(tmp_win, False);
+
+  return;
+}
+
+
 /***************************************************************************
  *
  * Handles destruction of a window
@@ -1942,7 +1992,6 @@ void free_window_names(FvwmWindow *tmp, Bool nukename, Bool nukeicon)
  ****************************************************************************/
 void destroy_window(FvwmWindow *tmp_win)
 {
-  extern FvwmWindow *ButtonWindow;
   extern Bool PPosOverride;
 
   /*
@@ -1953,6 +2002,24 @@ void destroy_window(FvwmWindow *tmp_win)
   if(!tmp_win)
     return;
 
+  /****** remove from window list ******/
+
+  /* first of all, remove the window from the list of all windows! */
+  tmp_win->prev->next = tmp_win->next;
+  if (tmp_win->next != NULL)
+    tmp_win->next->prev = tmp_win->prev;
+
+  /****** also remove it from the stack ring ******/
+
+  /*
+   * RBW - 11/13/1998 - new: have to unhook the stacking order chain also.
+   * There's always a prev and next, since this is a ring anchored on
+   * Scr.FvwmRoot
+   */
+  remove_window_from_stack_ring(tmp_win);
+
+  /****** check if we have to delay window destruction ******/
+
   if (Scr.flags.is_executing_complex_function && !DO_REUSE_DESTROYED(tmp_win))
   {
     /* mark window for destruction */
@@ -1973,26 +2040,13 @@ void destroy_window(FvwmWindow *tmp_win)
     {
       XUnmapWindow(dpy, tmp_win->frame);
     }
+    adjust_fvwm_internal_windows(tmp_win);
     BroadcastPacket(M_DESTROY_WINDOW, 3,
 		    tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
 
     return;
   }
 
-  /****** remove from window list ******/
-
-  /* first of all, remove the window from the list of all windows! */
-  /*
-      RBW - 11/13/1998 - new: have to unhook the stacking order chain also.
-      There's always a prev and next, since this is a ring anchored on
-      Scr.FvwmRoot
-  */
-  remove_window_from_stack_ring(tmp_win);
-
-  tmp_win->prev->next = tmp_win->next;
-  if (tmp_win->next != NULL)
-    tmp_win->next->prev = tmp_win->prev;
-
   /****** unmap the frame ******/
 
   XUnmapWindow(dpy, tmp_win->frame);
@@ -2000,22 +2054,18 @@ void destroy_window(FvwmWindow *tmp_win)
   if(!PPosOverride)
     XSync(dpy,0);
 
-  /****** broadcast ******/
+  /* already done above? */
+  if (!IS_SCHEDULED_FOR_DESTROY(tmp_win))
+  {
+    /****** adjust fvwm internal windows and the focus ******/
 
-  BroadcastPacket(M_DESTROY_WINDOW, 3,
-                  tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
+    adjust_fvwm_internal_windows(tmp_win);
 
-  /****** adjust fvwm internal windows ******/
+    /****** broadcast ******/
 
-  if (tmp_win == Scr.Hilite)
-    Scr.Hilite = NULL;
-  update_last_screen_focus_window(tmp_win);
-  if (ButtonWindow == tmp_win)
-    ButtonWindow = NULL;
-
-  /****** adjust focus ******/
-
-  restore_focus_after_unmap(tmp_win, False);
+    BroadcastPacket(M_DESTROY_WINDOW, 3,
+		    tmp_win->w, tmp_win->frame, (unsigned long)tmp_win);
+  }
 
   /****** adjust fvwm internal windows II ******/
 
@@ -2115,6 +2165,7 @@ void RestoreWithdrawnLocation(
   XWindowChanges xwc;
   rectangle naked_g;
   rectangle unshaded_g;
+  XSetWindowAttributes xswa;
 
   if(!tmp)
     return;
@@ -2174,6 +2225,11 @@ void RestoreWithdrawnLocation(
       }
     }
   }
+
+  /* restore initial backing store setting on window */
+  xswa.backing_store = tmp->initial_backing_store;
+  XChangeWindowAttributes(dpy, tmp->w, CWBackingStore, &xswa);
+  /* reparent to root window */
   XReparentWindow(
     dpy, tmp->w, (parent == None) ? Scr.Root : parent, xwc.x, xwc.y);
 
@@ -2202,18 +2258,19 @@ void RestoreWithdrawnLocation(
  ************************************************************************/
 void Reborder(void)
 {
-  FvwmWindow *tmp;			/* temp fvwm window structure */
+  FvwmWindow *tmp;
 
   /* put a border back around all windows */
   MyXGrabServer (dpy);
 
-  InstallWindowColormaps (&Scr.FvwmRoot);	/* force reinstall */
-/*
-    RBW - 05/15/1998
-    Grab the last window and work backwards: preserve stacking order on restart.
-*/
-  for (tmp = Scr.FvwmRoot.stack_prev; tmp != &Scr.FvwmRoot;
-       tmp = tmp->stack_prev)
+  /* force reinstall */
+  InstallWindowColormaps (&Scr.FvwmRoot);
+
+  /* RBW - 05/15/1998
+   * Grab the last window and work backwards: preserve stacking order on
+   * restart. */
+  for (tmp = get_prev_window_in_stack_ring(&Scr.FvwmRoot); tmp != &Scr.FvwmRoot;
+       tmp = get_prev_window_in_stack_ring(tmp))
   {
     if (!IS_ICONIFIED(tmp) && Scr.CurrentDesk != tmp->Desk)
     {
