@@ -636,7 +636,12 @@ static char *expand(
 	  }
 	  else
 	  {
-	    out[j++] = input[i];
+	    /* copy the whole string in square brackets */
+	    for ( ; i <= m; i++, j++)
+	    {
+		    out[j] = input[i];
+	    }
+	    i--;
 	  }
 	}
 	else
@@ -1247,7 +1252,7 @@ int DeferExecution(
 	MyXUngrabKeyboard(dpy);
 	return True;
       }
-      Keyboard_shortcuts(eventp, NULL, FinishEvent);
+      Keyboard_shortcuts(eventp, NULL, NULL, NULL, FinishEvent);
     }
     if(eventp->type == FinishEvent)
     {
