@@ -604,6 +604,9 @@ void DrawIconWindow(FvwmWindow *tmp_win)
       XConfigureWindow(dpy, tmp_win->icon_pixmap_w, mask, &xwc);
     }
   }
+  /* wait for pending EnterNotify/LeaveNotify events to suppress race condition
+   * w/ expanding/collapsing icon titles */
+  XSync(dpy, 0);
 }
 
 
@@ -629,7 +632,9 @@ void RedoIconName(FvwmWindow *tmp_win)
                  strlen(tmp_win->icon_name));
   /* clear the icon window, and trigger a re-draw via an expose event */
   if (IS_ICONIFIED(tmp_win))
+  {
     DrawIconWindow(tmp_win);
+  }
   if (IS_ICONIFIED(tmp_win))
     XClearArea(dpy, tmp_win->icon_w, 0, 0, 0, 0, True);
   return;
@@ -652,6 +657,7 @@ void AutoPlaceIcon(FvwmWindow *t)
   FvwmWindow *test_window;
   Bool loc_ok;
   Bool loc_ok_wrong_screen;
+  Bool loc_ok_wrong_screen2;
   int real_x=10, real_y=10;
   int new_x, new_y;
   Bool do_move_pixmap = False;
@@ -784,8 +790,8 @@ void AutoPlaceIcon(FvwmWindow *t)
 #define HRZ_FILL icon_boxes_ptr->IconFlags & ICONFILLHRZ
 
     /* needed later */
-    fscr.xypos.x = t->frame_g.x + t->frame_g.width / 2;
-    fscr.xypos.y = t->frame_g.y + t->frame_g.height / 2;
+    fscr.xypos.x = t->frame_g.x + (t->frame_g.width / 2) - base_x;
+    fscr.xypos.y = t->frame_g.y + (t->frame_g.height / 2) - base_y;
     /* unnecessary copy of width */
     width = t->icon_p_width;
     /* total height */
@@ -804,8 +810,11 @@ void AutoPlaceIcon(FvwmWindow *t)
         break;
       }
       /* get the screen dimensions for the icon box */
+      if (icon_boxes_ptr->IconScreen == FSCREEN_CURRENT)
+	fscr.mouse_ev = NULL;
       FScreenGetScrRect(
-	NULL, icon_boxes_ptr->IconScreen,
+	&fscr,
+        icon_boxes_ptr->IconScreen,
 	&ref.x, &ref.y, &ref.width, &ref.height);
       dim[1].screen_offset = ref.y;
       dim[1].screen_dimension = ref.height;
@@ -894,12 +903,14 @@ void AutoPlaceIcon(FvwmWindow *t)
       } /* end horizontal dimension first */
       /* save for reseting inner loop */
       dim[0].start_at = dim[2].start_at;
+      loc_ok_wrong_screen2 = False;
       while((dim[1].step < 0            /* filling reversed */
              ? (dim[1].start_at + dim[1].icon_dimension - dim[1].nom_dimension
                 > dim[1].end_at)        /* check back edge */
              : (dim[1].start_at + dim[1].nom_dimension
                 < dim[1].end_at))       /* check front edge */
-            && (!loc_ok)) {             /* nothing found yet */
+            && (!loc_ok)
+	    && (!loc_ok_wrong_screen2)) {  /* nothing found yet */
         dim[1].real_start = dim[1].start_at; /* init */
         if (dim[1].start_at + dim[1].icon_dimension >
 	    dim[1].screen_offset + dim[1].screen_dimension - 2 + dim[1].base)
@@ -920,7 +931,8 @@ void AutoPlaceIcon(FvwmWindow *t)
                   > dim[2].end_at)        /* check back edge */
                : (dim[2].start_at + dim[2].nom_dimension
                   < dim[2].end_at))       /* check front edge */
-              && (!loc_ok)) {             /* nothing found yet */
+	      && (!loc_ok)
+	      && (!loc_ok_wrong_screen2)) { /* nothing found yet */
           dim[2].real_start = dim[2].start_at; /* init */
           if (dim[2].start_at + dim[2].icon_dimension >
               dim[2].screen_offset + dim[2].screen_dimension - 2 + dim[2].base)
@@ -951,12 +963,16 @@ void AutoPlaceIcon(FvwmWindow *t)
 
 	  /* this may be a good location */
 	  if (FScreenIsRectangleOnScreen(&fscr, FSCREEN_XYPOS, &ref))
+	  {
 	    loc_ok = True;
+	  }
 	  else
-	    loc_ok_wrong_screen = True;
+	  {
+	    loc_ok_wrong_screen2 = True;
+	  }
           test_window = Scr.FvwmRoot.next;
           while((test_window != (FvwmWindow *)0)
-                &&(loc_ok == True || loc_ok_wrong_screen))
+                &&(loc_ok == True || loc_ok_wrong_screen2))
 	  {
 	    /* test overlap */
             if(test_window->Desk == t->Desk)
@@ -977,12 +993,16 @@ void AutoPlaceIcon(FvwmWindow *t)
 		{
 		  /* don't accept this location */
                   loc_ok = False;
-                  loc_ok_wrong_screen = False;
+                  loc_ok_wrong_screen2 = False;
                 } /* end if icons overlap */
               } /* end if its an icon */
             } /* end if same desk */
             test_window = test_window->next;
           } /* end while icons that may overlap */
+	  if (loc_ok_wrong_screen2)
+	  {
+	    loc_ok_wrong_screen = True;
+	  }
 	  /* Grid inner value & direction */
           dim[2].start_at += dim[2].step;
         } /* end while room inner dimension */
@@ -1168,12 +1188,18 @@ void GetIconWindow(FvwmWindow *tmp_win)
 {
   /* We are guaranteed that wmhints is non-null when calling this
    * routine */
-  if(XGetGeometry(dpy,   tmp_win->wmhints->icon_window, &JunkRoot,
-		  &JunkX, &JunkY,(unsigned int *)&tmp_win->icon_p_width,
-		  (unsigned int *)&tmp_win->icon_p_height,
-		  &JunkBW, &JunkDepth)==0)
+  if (XGetGeometry(
+	dpy, tmp_win->wmhints->icon_window, &JunkRoot, &JunkX, &JunkY,
+	(unsigned int *)&tmp_win->icon_p_width,
+	(unsigned int *)&tmp_win->icon_p_height, &JunkBW, &JunkDepth) == 0)
   {
-    fvwm_msg(ERR,"GetIconWindow","Help! Bad Icon Window!");
+    fvwm_msg(ERR,"GetIconWindow", "Window '%s' has a bad icon window!"
+	     " Ignoring icon window.",
+	     tmp_win->name);
+    /* disable the icon window hint */
+    tmp_win->wmhints->icon_window = None;
+    tmp_win->wmhints->flags &= ~IconWindowHint;
+    return;
   }
   tmp_win->icon_p_width += JunkBW<<1;
   tmp_win->icon_p_height += JunkBW<<1;
@@ -1212,13 +1238,24 @@ void GetIconBitmap(FvwmWindow *tmp_win)
   if (!XGetGeometry(dpy, tmp_win->wmhints->icon_pixmap, &JunkRoot,
 		    &JunkX, &JunkY, &width, &height, &JunkBW, &depth))
   {
+    fvwm_msg(ERR,"GetIconBitmap", "Window '%s' has a bad icon pixmap!"
+	     " Ignoring icon.", tmp_win->name);
+    /* disable icon pixmap hint */
+    tmp_win->wmhints->icon_pixmap = None;
+    tmp_win->wmhints->flags &= ~IconPixmapHint;
     return;
   }
 
   /* sanity check the pixmap depth, it must be the same as the root or 1 */
   if ((depth != 1) && (depth != DefaultDepth(dpy,Scr.screen)))
   {
-    fvwm_msg(ERR, "GetIconBitmap", "Bad client icon pixmap depth %d", depth);
+    fvwm_msg(ERR, "GetIconBitmap",
+	     "Window '%s' has a bad icon bitmap depth %d (should be 1 or %d)!"
+	     " Ignoring icon bitmap.",
+	     tmp_win->name, depth, DefaultDepth(dpy,Scr.screen));
+    /* disable icon pixmap hint */
+    tmp_win->wmhints->icon_pixmap = None;
+    tmp_win->wmhints->flags &= ~IconPixmapHint;
     return;
   }
 
