@@ -24,6 +24,7 @@
 #if HAVE_UNAME
 #include <sys/utsname.h>
 #endif
+#include <sys/stat.h>
 
 
 /*
@@ -73,15 +74,13 @@ void setPath( char** p_path, const char* newpath, int free_old_path )
     int found_plus = strchr( newpath, '+' ) != NULL;
 
     /** Leave room for the old path, if we find a '+' in newpath **/
-    *p_path = envDupExpand( stripped_path, found_plus ? oldlen : 0 );
+    *p_path = envDupExpand( stripped_path, found_plus ? oldlen - 1 : 0 );
     free( stripped_path );
 
     if ( found_plus ) {
 	char* p = strchr( *p_path, '+' );
-	memmove( p+oldlen, p+1, strlen(p+1) );
-
-	/* copy oldlen+1 bytes to include the trailing NUL */
-	strncpy( p, oldpath, oldlen+1 );
+	memmove(p + oldlen, p + 1, strlen(p+1) + 1);
+	memmove(p, oldpath, oldlen);
     }
 
     if ( free_old_path )
@@ -155,3 +154,36 @@ char* searchPath( const char* pathlist, const char* filename,
     free(path);
     return NULL;
 }
+
+/*
+ * void setFileStamp(FileStamp *stamp, const char *name);
+ * Bool isFileStampChanged(const FileStamp *stamp, const char *name);
+ *
+ * An interface for verifying cached files.
+ * The first function associates a file stamp with file (named by name).
+ * The second function returns True or False in case the file was changed
+ * from the time the stamp was associated.
+ *
+ * FileStamp can be a structure; try to save memory by evaluating a checksum.
+ */
+
+FileStamp getFileStamp(const char *name)
+{
+  static struct stat buf;
+
+  if (!name || stat(name, &buf))
+    return 0;
+  return
+    ((FileStamp)buf.st_mtime << 13) + (FileStamp)buf.st_size;
+}
+
+void setFileStamp(FileStamp *stamp, const char *name)
+{
+  *stamp = getFileStamp(name);
+}
+
+Bool isFileStampChanged(const FileStamp *stamp, const char *name)
+{
+  return
+    *stamp != getFileStamp(name);
+}
